/* 
  CESYS Software License - Version 1.0 - January 1st, 2016

  Permission is hereby granted, free of charge, to any person or organization
  obtaining a copy of the software and accompanying documentation covered by
  this license (the "Software") to use, reproduce, display, distribute,
  execute, and transmit the Software, and to prepare derivative works of the
  Software, and to permit third-parties to whom the Software is furnished to
  do so, all subject to the following:

  The software must only be used to operate hardware manufactured by 
  CESYS GmbH, Herzogenaurach.

  The copyright notices in the Software and this entire statement, including
  the above license grant, this restriction and the following disclaimer,
  must be included in all copies of the Software, in whole or in part, and
  all derivative works of the Software, unless such copies or derivative
  works are solely in the form of machine-executable object code generated by
  a source language processor.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
  SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
  FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
  
  Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2018
*/  

#include <stdexcept>
#include <algorithm>

#include "beastlink++.h"

namespace beastlink {

inline void check(_c::BL_RESULT res) {
	if (res != _c::BL_SUCCESS) {
		std::vector<char> tmp(_c::BL_IF_BUFFER_SIZE, 0);
		_c::BlGetLastErrorText(res, &tmp[0]);
		throw std::runtime_error(&tmp[0]);
	}
}

static int _enumRound = 0;

void DeviceInfo::setUserId(const std::string &userId) {
	if (_handle == _c::BL_INVALID_DEVICE)
		throw std::runtime_error("Device not opened.");

	std::vector<char> tmp(_c::BL_IF_BUFFER_SIZE, 0);
	std::copy(
		userId.begin(), 
		userId.begin() + std::min<size_t>(userId.size(), _c::BL_IF_BUFFER_SIZE), 
		tmp.begin());
	check(_c::BlSetUserId(_handle, &tmp[0]));
}

std::string DeviceInfo::getUserId() const { 
	if (_handle == _c::BL_INVALID_DEVICE)
		return "";

	try {
		std::vector<char> tmp(_c::BL_IF_BUFFER_SIZE, 0);
		check(_c::BlGetUserId(_handle, &tmp[0]));
		return std::string(&tmp[0]);
	} catch (std::exception &) { return ""; }
}

std::string DeviceInfo::getSerialNumber() const { 
	if (_handle == _c::BL_INVALID_DEVICE)
		return "";

	try {
		std::vector<char> tmp(_c::BL_IF_BUFFER_SIZE, 0);
		check(_c::BlGetSerialNumber(_handle, &tmp[0]));
		return std::string(&tmp[0]);
	} catch (std::exception &) { return ""; }
}

Version DeviceInfo::getFirmwareVersion() const { 
	Version v;
	if (_handle != _c::BL_INVALID_DEVICE) {
		try {
			check(_c::BlGetFirmwareVersion(_handle, &v.major, &v.minor, &v.fix));
		} catch (std::exception &) {}
	}

	return v;
}

std::string DeviceInfo::getDerivateInfo() const { 
	if (_handle == _c::BL_INVALID_DEVICE)
		return "";

	try {
		std::vector<char> tmp(_c::BL_IF_BUFFER_SIZE, 0);
		check(_c::BlGetDerivateInfo(_handle, &tmp[0]));
		return std::string(&tmp[0]);
	} catch (std::exception &) { return ""; }
}

uint32_t DeviceInfo::getDerivateId() const {
	if (_handle == _c::BL_INVALID_DEVICE)
		return 0xffffffff;

	try {
		uint32_t did = 0;
		check(_c::BlGetDerivateId(_handle, &did)); 
		return did;
	} catch (std::exception &) { return 0xffffffff; }
}

void Device::close() { 
	if (!_closed) {
		// Postpone error checking to safely clean up field vars.
		auto ec = _c::BlClose(_handle);
		_handle = _c::BL_INVALID_DEVICE;
		_closed = true;
		check(ec);
	}
}

unsigned int Device::readRegister(uint32_t reg) {
	uint32_t r;
	check(_c::BlReadRegister(_handle, reg, &r));
	return r;
}

void Device::writeRegister(uint32_t reg, uint32_t value) {
	check(_c::BlWriteRegister(_handle, reg, value)); 
}

void Device::readBlock(uint32_t addr, void *buffer, int size, int flags) {
	check(_c::BlReadBlock(_handle, addr, buffer, size, flags)); 
}

void Device::readBlock(uint32_t addr, std::vector<uint8_t> &buffer, int flags) {
	check(_c::BlReadBlock(_handle, addr, buffer.data(), int(buffer.size()), flags)); 
}

std::vector<uint8_t> Device::readBlock(uint32_t addr, size_t bytes, int flags) {
	std::vector<uint8_t> buffer(bytes);
	readBlock(addr, buffer, flags);
	return buffer;
}

void Device::writeBlock(uint32_t addr, void *buffer, int size, int flags) {
	check(_c::BlWriteBlock(_handle, addr, buffer, size, flags)); 
}

void Device::writeBlock(uint32_t addr, const std::vector<uint8_t> &buffer, int flags) {
	check(_c::BlWriteBlock(_handle, addr, const_cast<uint8_t*>(buffer.data()), int(buffer.size()), flags));
}

void Device::resetFpga() { 
	check(_c::BlResetFpga(_handle)); 
}

void Device::programFpgaFromBin(const std::string &fileName) { 
	check(_c::BlProgramFpgaFromBin(_handle, fileName.c_str()));
}

void Device::programFpgaFromMemory(void *data, int size, bool compressed) { 
	check(_c::BlProgramFpgaFromMemory(_handle, data, size, compressed ? 1 : 0)); 
}

void Device::setTimeout(uint32_t timeout) {
	check(_c::BlSetTimeout(_handle, timeout)); 
}

Device EnumeratedDevice::open(bool autoClose) const { 
	if (_enumRound != _erid)
		throw std::runtime_error("Invalid enumeration round.");

	auto handle = _c::BL_INVALID_DEVICE;
	check(_c::BlOpen(_eid, &handle)); 

	return Device(_info, handle, autoClose);
}

DevicePtr EnumeratedDevice::openAsPtr(bool autoClose) const {
	if (_enumRound != _erid)
		throw std::runtime_error("Invalid enumeration round.");

	auto handle = _c::BL_INVALID_DEVICE;
	check(_c::BlOpen(_eid, &handle));

	//return std::make_shared<Device>(_info, handle, autoClose);
	return std::shared_ptr<Device>(new Device(_info, handle, autoClose));
}

Version getVersion() { 
	Version v;
	check(_c::BlGetVersion(&v.major, &v.minor, &v.fix));
	return v;
}

void init() { check(_c::BlInit()); }

std::vector<EnumeratedDevice> enumerate(uint16_t vid, uint16_t pid) {
	++_enumRound;

	int count = 0;
	check(_c::BlEnumerate(vid, pid, &count));

	std::vector<EnumeratedDevice> r;
	for (int i = 0; i < count; ++i) {
		r.push_back(EnumeratedDevice(DeviceInfo(vid, pid), i, _enumRound));
	}

	return r;
}

void cleanup() { 
	check(_c::BlCleanup()); 
}

void setLogLevel(int ll) {
	check(_c::BlSetLogLevel(ll));
}

} // namespace beastlink
