/*
  CESYS Software License - Version 1.0 - January 1st, 2016

  Permission is hereby granted, free of charge, to any person or organization
  obtaining a copy of the software and accompanying documentation covered by
  this license (the "Software") to use, reproduce, display, distribute,
  execute, and transmit the Software, and to prepare derivative works of the
  Software, and to permit third-parties to whom the Software is furnished to
  do so, all subject to the following:

  The software must only be used to operate hardware manufactured by
  CESYS GmbH, Herzogenaurach.

  The copyright notices in the Software and this entire statement, including
  the above license grant, this restriction and the following disclaimer,
  must be included in all copies of the Software, in whole or in part, and
  all derivative works of the Software, unless such copies or derivative
  works are solely in the form of machine-executable object code generated by
  a source language processor.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
  SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
  FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2018
*/

/*
 This example demonstrates most features of the BEASTLINK C API.
*/

/* Include the C-Library */
#include <beastlink.h>

/* Some standard libraries. */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Used for addresses and register values. */
#if !defined(uint32_t)
typedef unsigned int uint32_t;
#endif
#if !defined(uint8_t)
typedef unsigned char uint8_t;
#endif

// We use VID and PID for EFM-03 devices.
const int VID = 0x10f8;
const int PID = 0xc583;

/* Helper function that reads the error text and forward it to stdout. */
void printError(BL_RESULT code) {
	char error[BL_IF_BUFFER_SIZE];

	if (code != BL_SUCCESS) {
		BlGetLastErrorText((BL_RESULT)code, error);
		printf("Error, terminate application: %s\n", error);
	}
}

/*
 Do a device/FPGA design example specific test for each
 device that ships with an example design.
*/
BL_RESULT deviceTest() {
	BL_RESULT result = BL_SUCCESS;
	int deviceCount = -1, i = 0, l = 0, *p = 0;
	BL_DEVICE_HANDLE handle = BL_INVALID_DEVICE;
	// The example maps 16K Block RAM onto the AXI-4 Bus, starting at address 0x00000000.
	uint32_t BlockRAMAddr = 0x00000000;
	int BlockRAMSize = 16 * 1024;
	int HalfRAMSize = BlockRAMSize / 2;
	uint8_t *data = NULL, *rbdata = NULL;

	/*
	 Start enumeration, the second parameter returns the count of
	 devices found on the system.
	*/
	result = BlEnumerate(VID, PID, &deviceCount);
	if (result != BL_SUCCESS) return result;

	/* Loop over all devices. */
	for (i = 0; i < deviceCount; ++i) {
		/* Open device. */
		result = BlOpen(i, &handle);
		if (result != BL_SUCCESS) goto Error;

		/* This is the example design by Cesys, the design is read from
		   the specified file and programmed to the FPGA on the device (must be .bin format).
		   Optionally, BlProgramFpgaFromMemory() can
		   be used to download a compiled in resource (generated by tools).
		   BlResetFpga() is explicitly invoked from within this call. */
		result = BlProgramFpgaFromBin(handle, "./beastlink_exdes_top.bin");
		if (result != BL_SUCCESS) goto Error;

		/* Create 16kb block containing counting 32 bit values. */
		data = (uint8_t*)malloc(BlockRAMSize);
		p = (int*)data;
		for (l = 0; l < BlockRAMSize / (int)sizeof(int); ++l)
			*p++ = l;

		/* Write to Block RAM. */
		result = BlWriteBlock(handle, BlockRAMAddr, data, BlockRAMSize, 0);
		if (result != BL_SUCCESS) goto Error;

		/* Read back, starting at 8k. */
		rbdata = (uint8_t*)malloc(HalfRAMSize);
		result = BlReadBlock(handle, BlockRAMAddr + HalfRAMSize, rbdata, HalfRAMSize, 0);

		/* Write compare result. */
		printf("Sequences are %s.",
			memcmp(data + HalfRAMSize, rbdata, HalfRAMSize) ? "not equal" : "equal");

Error:

		/* Free allocated areas. */
		free(rbdata);
		free(data);

		/* Close device. */
		BlClose(handle);
	}

	return result;
}

/* Enumerate all known devices and write information to stdout. */
BL_RESULT infoTest() {
	BL_RESULT result = BL_SUCCESS;
	int deviceCount = -1, i = 0;
	int fwmajor = 0, fwminor = 0, fwfix = 0;
	char userId[BL_IF_BUFFER_SIZE] = "";
	BL_DEVICE_HANDLE handle = BL_INVALID_DEVICE;
	char serial[BL_IF_BUFFER_SIZE] = "", dinf[BL_IF_BUFFER_SIZE] = "";
	uint32_t did = 0xffffffff;

	/*
	 Start enumeration, the parameter returns the count of
	 devices found on the system.
	*/
	result = BlEnumerate(VID, PID, &deviceCount);
	if (result != BL_SUCCESS) return result;

	/* Loop over all devices. */
	for (i = 0; i < deviceCount; ++i) {
		/* Get information about the current enumeration. */
		/* Open device. */
		result = BlOpen(i, &handle);
		if (result != BL_SUCCESS) return result;

		/* Write all information to stdout, values are self explaining. */
		printf("Device found:\n");
		BlGetUserId(handle, userId);
		printf("  UserId          : %s\n", userId);
		BlGetSerialNumber(handle, serial);
		printf("  SerialNumber    : %s\n", serial);
		BlGetFirmwareVersion(handle, &fwmajor, &fwminor, &fwfix);
		printf("  FirmwareVersion : %i.%i.%i\n", fwmajor, fwminor, fwfix);
		BlGetDerivateInfo(handle, dinf);
		printf("  DerivateInfo    : %s\n", dinf);
		BlGetDerivateId(handle, &did);
		printf("  DerivateId      : 0x%08x\n", did);

		/* We're done here. Close the device. */
		result = BlClose(handle);
		if (result != BL_SUCCESS) return result;
	}

	return BL_SUCCESS;
}

/* Application entry point. */
int main() {
	BL_RESULT result = BL_SUCCESS;
	int major = 0, minor = 0, fix = 0;

	/* Initialize BeastLink before using it. */
	result = BlInit();
	if (result != BL_SUCCESS) {
		printError(result);
		return -1;
	}

	/* Output BeastLink version to stdout. */
	BlGetVersion(&major, &minor, &fix);
	if (result != BL_SUCCESS) {
		printError(result);
		return -1;
	}
	printf("Beastlink version: %i.%i.%i\n", major, minor, fix);

	/*
	 Set log level to log up to warning level (default)
	 Valid log levels are:
	 - 0x00: Off
	 - 0x01: Error
	 - 0x02: Warning
	 - 0x03: Debug
	*/
	result = BlSetLogLevel(0x2);
	if (result != BL_SUCCESS) {
		printError(result);
		return -1;
	}

	/*
	 Write all informations about all known connected
	 devices to stdout.
	*/
	result = infoTest();
	if (result != BL_SUCCESS) {
		printError(result);
		return -1;
	}

	/*
	 Do a device specific test (with devices that have an example FPGA design).
	*/
	result = deviceTest();
	if (result != BL_SUCCESS) {
		printError(result);
		return -1;
	}

	/*
	 Cleanup internal structures. This closes all
	 opened resources, no further use of the BeastLink API
	 is possible until BlInit() is called again.
	*/
	BlCleanup();

	return 0;
}
