﻿/* 
  CESYS Software License - Version 1.0 - January 1st, 2016

  Permission is hereby granted, free of charge, to any person or organization
  obtaining a copy of the software and accompanying documentation covered by
  this license (the "Software") to use, reproduce, display, distribute,
  execute, and transmit the Software, and to prepare derivative works of the
  Software, and to permit third-parties to whom the Software is furnished to
  do so, all subject to the following:

  The software must only be used to operate hardware manufactured by 
  CESYS GmbH, Herzogenaurach.

  The copyright notices in the Software and this entire statement, including
  the above license grant, this restriction and the following disclaimer,
  must be included in all copies of the Software, in whole or in part, and
  all derivative works of the Software, unless such copies or derivative
  works are solely in the form of machine-executable object code generated by
  a source language processor.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
  SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
  FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
  
  Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2018
*/  

using System;

namespace BeastLink
{
    /// <summary>
    /// This type offers functionality to communicate with a single device. Instances can be
    /// retrieved from EnumeratedDevice.Open() and represent an opened connection to a device.
    /// </summary>
    public sealed class Device : IDisposable
    {
        private readonly int _handle;

        internal Device(DeviceInfo info, int handle)
        {
            info._handle = handle;
            DeviceInfo = info;
            _handle = handle;
        }

        /// <summary>
        /// This flag can be used in ReadBlock() and WriteBlock() to specify transfer to a 
        /// constant address. The opposite and *default* behavior is incremental address.
        /// Attention: For historical reasons, this flag is interpreted inverse inside the FPGA interface ->
        /// "address auto increment" is 0 if FlagConstAddress is specified.
        /// </summary>
        public readonly static int FlagConstAddr = SystemConnector.BL_FLAG_CONST_ADDR;

        /// <summary>
        /// Get general device info.
        /// </summary>
        public DeviceInfo DeviceInfo { get; private set; }

        /// <summary>
        /// Free access to device. After this call the device is available in new enumerations and
        /// this instance of type Device can't be used anymore.
        /// </summary>
        public void Close()
        {
            SystemConnector.BlClose(_handle);
        }

        /// <summary>
        /// Read 32 bit register value.
        /// </summary>
        /// <param name="reg">Register in FPGA address space.</param>
        /// <returns>Value of register.</returns>
        public uint ReadRegister(uint reg)
        {
            return SystemConnector.BlReadRegister(_handle, reg);
        }

        /// <summary>
        /// Write 32 bit register value.
        /// </summary>
        /// <param name="reg">Register in FPGA address space.</param>
        /// <param name="value">Value to write to register.</param>
        public void WriteRegister(uint reg, uint value)
        {
            SystemConnector.BlWriteRegister(_handle, reg, value);
        }

        /// <summary>
        /// Read data block.
        /// </summary>
        /// <param name="addr">Start address in FPGA address space.</param>
        /// <param name="buffer">Buffer to read to from FPGA address space.</param>
        /// <param name="flags">I/O specific flags (see their individual documentation)
        ///              - Device.FlagConstAddress</param>
        public void ReadBlock(uint addr, byte[] buffer, int flags = 0)
        {
            SystemConnector.BlReadBlock(_handle, addr, buffer, flags);
        }

        /// <summary>
        /// Read data block.
        /// </summary>
        /// <param name="addr">Start address in FPGA address space.</param>
        /// <param name="bytes">Count of bytes to read.</param>
        /// <param name="flags">I/O specific flags (see their individual documentation)
        ///              - Device.FlagConstAddress</param>
        /// <returns>Read block.</returns>
        public byte[] ReadBlock(uint addr, int bytes, int flags = 0) 
        {
            var r = new byte[bytes];
            ReadBlock(addr, r, flags);
            return r;
        }

        /// <summary>
        /// Write data block.
        /// </summary>
        /// <param name="addr">Start address in FPGA address space.</param>
        /// <param name="buffer">Data to write to FPGA address space.</param>
        /// <param name="flags">I/O specific flags (see their individual documentation)
        ///              - Device.FlagConstAddress</param>
        public void WriteBlock(uint addr, byte[] buffer, int flags = 0)
        {
            SystemConnector.BlWriteBlock(_handle, addr, buffer, flags);
        }

        /// <summary>
        /// Pulse FPGA reset line (the I/O that is chosen to be reset). Useful to bring the FPGA
        /// in a defined state. Implicitly done after FPGA programming.
        /// </summary>
        public void ResetFpga()
        {
            SystemConnector.BlResetFpga(_handle);
        }

        /// <summary>
        /// Configure FPGA with given design in file. Loads a .bin from as specified by 
        /// fileName and programs the FPGA using this design. Subsequently pulses the reset line.
        /// </summary>
        /// <param name="fileName">File which contains design file data.</param>
        public void ProgramFpgaFromBin(string fileName)
        {
            SystemConnector.BlProgramFpgaFromBin(_handle, fileName);
        }

        /// <summary>
        /// Configure FPGA with given design in memory. Useful when compiling FPGA designs into 
        /// the executable. UDK3BoardManager creates sources that can be used this way. 
        /// It is preferred to use ProgramFpgaFromMemoryZ().
        /// Subsequently pulses the reset line.
        /// </summary>
        /// <param name="buffer">Design buffer.</param>
        public void ProgramFpgaFromMemory(byte[] buffer, bool compressed)
        {
            SystemConnector.BlProgramFpgaFromMemory(_handle, buffer, compressed);
        }

        /// <summary>
        /// Configure timeout until a pending transfer is stopped and defined as failed.
        /// This timeout is used for all transfers to and from device.
        /// </summary>
        /// <param name="timeout">Time span in milliseconds.</param>
        public void SetTimeout(int timeout)
        {
            SystemConnector.BlSetTimeout(_handle, timeout);
        }

        /// <summary>
        /// Implement IDisposable.
        /// </summary>
        public void Dispose() {
            Close();
        }
    }
}
