﻿/* 
  CESYS Software License - Version 1.0 - January 1st, 2016

  Permission is hereby granted, free of charge, to any person or organization
  obtaining a copy of the software and accompanying documentation covered by
  this license (the "Software") to use, reproduce, display, distribute,
  execute, and transmit the Software, and to prepare derivative works of the
  Software, and to permit third-parties to whom the Software is furnished to
  do so, all subject to the following:

  The software must only be used to operate hardware manufactured by 
  CESYS GmbH, Herzogenaurach.

  The copyright notices in the Software and this entire statement, including
  the above license grant, this restriction and the following disclaimer,
  must be included in all copies of the Software, in whole or in part, and
  all derivative works of the Software, unless such copies or derivative
  works are solely in the form of machine-executable object code generated by
  a source language processor.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
  SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
  FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
  
  Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2018
*/  

using System.IO;

namespace BeastLink
{
    /// <summary>
    /// This type offers access to most of the device information. Some information
    /// is available after opening the device. An instance to this type can either
    /// be retrieved from EnumeratedDevice.DeviceInfo or Device.DeviceInfo.
    /// </summary>
    public sealed class DeviceInfo
    {
        internal int _handle = SystemConnector.BL_INVALID_DEVICE;

        internal DeviceInfo(int vid, int pid)
        {
            Vid = vid;
            Pid = pid;
        }

        /// <summary>
        /// Vendor ID of device (if applicable).
        /// </summary>
        public int Vid { get; private set; }

        /// <summary>
        /// Product ID of device (if applicable).
        /// </summary>
        public int Pid { get; private set; }
        
        /// <summary>
        /// Get / Set user ID. Writes the string to the device EEPROM or reads it from there.
        /// This ID is intended to differentiate multiple devices of the same type connected
        /// to one host. The ID can contain up to 63 ASCII characters.
        /// Returns "" if not accessible or in case of error.
        /// </summary>
        public string UserId
        {
            get
            {
                if (_handle == SystemConnector.BL_INVALID_DEVICE)
                    return "";

                try
                {
                    return SystemConnector.BlGetUserId(_handle);
                }
                catch (IOException) { return ""; }
            }

            set
            {
                if (_handle == SystemConnector.BL_INVALID_DEVICE)
                    throw new IOException("Device not opened.");

                SystemConnector.BlSetUserId(_handle, value);
            }
        }


        /// <summary>
        /// Return serial number. Not supported by all device 
        /// types / instances. "" if not accessible or in case of error.
        /// </summary>
        public string SerialNumber
        {
            get
            {
                if (_handle == SystemConnector.BL_INVALID_DEVICE)
                    return "";

                try
                {
                    return SystemConnector.BlGetSerialNumber(_handle);
                }
                catch (IOException) { return ""; }
            }
        }

        /// <summary>
        /// Firmware version.
        /// </summary>
        public Version FirmwareVersion
        {
            get
            {
                if (_handle == SystemConnector.BL_INVALID_DEVICE)
                    return new Version(0, 0, 0);

                try
                {
                    int major = 0, minor = 0, fix = 0;
                    SystemConnector.BlGetFirmwareVersion(_handle, ref major, ref minor, ref fix);
                    return new Version(major, minor, fix);
                }
                catch (IOException) { return new Version(0, 0, 0); }
            }
        }

        /// <summary>
        /// Return derivate info string. "" if not accessible or in case of error.
        /// Some devices are available in different versions (different FPGA's, different peripherals).
        /// The returned string contains a textual information about the specific
        /// derivate. For programmatically differentiation, DerivateId is better suited.
        /// </summary>
        public string DerivateInfo
        {
            get
            {
                if (_handle == SystemConnector.BL_INVALID_DEVICE)
                    return "";

                try
                {
                    return SystemConnector.BlGetDerivateInfo(_handle);
                }
                catch (IOException) { return ""; }
            }
        }

        /// <summary>
        /// Return derivate id. 0xffffffff if not accessible or in case of error.
        /// This ID is unique for every derivate of a specific device type. 
        /// Existing derivates can be found in the documentation of the specific device type.
        /// </summary>
        public uint DerivateId
        {
            get
            {
                if (_handle == SystemConnector.BL_INVALID_DEVICE)
                    return 0xffffffff;
                try
                {
                    return SystemConnector.BlGetDerivateId(_handle);
                }
                catch (IOException) { return 0xffffffff; }
            }
        }
    }
}
